@@ -27,8 +27,8 @@ def relative_normpath(f: Optional[Text], path: Text) -> Optional[Text]:
         return None
 
 
-def list_to_str(l: List[Text], delim: Text = ", ", quote: Text = "'") -> Text:
-    return delim.join([quote + e + quote for e in l])
+def list_to_str(lst: List[Text], delim: Text = ", ", quote: Text = "'") -> Text:
+    return delim.join([quote + e + quote for e in lst])
 
 
 def ordered(obj: Any) -> Any:
